/* --COPYRIGHT--,BSD
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//******************************************************************************

//******************************************************************************

#include "driverlib.h"
#include <msp430.h>
#include "Board.h"
#include "Defines.h"
#include "main.h"

#define SLAVE_ADDR 0x01
#define INTER_BYTE_DELAY 5600  // at least 100 us
#define I2C_TIMEOUT 100000

unsigned long pre_eeprom_write_regs[24] = {0};
unsigned long post_eeprom_write_regs[24] = {0};

// User Configured regmap
unsigned long eeprom_regmap[24][2] = {
    {0x80, 0x64738C20},  // ISD_CONFIG
    {0x82, 0xA8200000},  // REV_DRIVE_CONFIG
    {0x84, 0x0B6807D0},  // MOTOR_STARTUP1
    {0x86, 0xA306600C},  // MOTOR_STARTUP2
    {0x88, 0x0D3201B5},  // CLOSED_LOOP1
    {0x8A, 0x9BAD0000},  // CLOSED_LOOP2
    {0x8C, 0x00000000},  // CLOSED_LOOP3
    {0x8E, 0x00000000},  // CLOSED_LOOP4
    {0x90, 0xBEC80106},  // FAULT_CONFIG1
    {0x92, 0xF0D00888},  // FAULT_CONFIG2
    {0x94, 0x00000000},  // SPEED_PROFILES1
    {0x96, 0x00000000},  // SPEED_PROFILES2
    {0x98, 0x00000000},  // SPEED_PROFILES3
    {0x9A, 0x800D0000},  // SPEED_PROFILES4
    {0x9C, 0x00000000},  // SPEED_PROFILES5
    {0x9E, 0x00000000},  // SPEED_PROFILES6
    {0xA0, 0xA433407D},  // INT_ALGO_1
    {0xA2, 0x000001A7},  // INT_ALGO_2
    {0xA4, 0x00000000},  // PIN_CONFIG
    {0xA6, 0x00101462},  // DEVICE_CONFIG1
    {0xA8, 0xC000F00F},  // DEVICE_CONFIG2
    {0xAA, 0xC1C01F00},  // PERI_CONFIG1
    {0xAC, 0x9C450100},  // GD_CONFIG1
    {0xAE, 0x80200000},  // GD_CONFIG2
};

void main(void)
{
    InitSystem();

    unsigned int i;

    /* Read the initial EEPROM registers into pre_eeprom_write_regs array */
    for (i = 0; i < 24; i++)
    {
        if(!I2C_read32(eeprom_regmap[i][0], &(pre_eeprom_write_regs[i]), I2C_TIMEOUT))
            while(1);
    }

    __no_operation();

    /* Write the user configured EEPROM registers to the shadow register */
    for (i = 0; i < 24; i++)
    {
        if (!I2C_write32(eeprom_regmap[i][0], eeprom_regmap[i][1], I2C_TIMEOUT))
            while(1);
    }

    __no_operation();  // For setting breakpoint while debugging

    /* Set the EEPROM_WRITE bit along with EEPROM_WRITE_ACCESS_KEY in the DEV_CTRL register (0xEA located in RAM)
     * This copies the content of the shadow registers into the EERPOM memory
     */
    if (!I2C_write32(0xEA, 0x8A500000, I2C_TIMEOUT))
            while(1);

    __delay_cycles(2000000); // delay at least 100ms for copy operation from Shadow-register to EEPROM to complete
    __no_operation(); // For setting breakpoint while debugging

    /* Set the EEPROM_READ bit in the DEV_CTRL register (0xEA located in RAM
     * This copies the content of the EEPROM memory back into the shadow registers
     */
    if (!I2C_write32(0xEA, 0x40000000, I2C_TIMEOUT))
            while(1);

    __delay_cycles(2000000); // delay at least 100ms for copy operation from EEPROM to Shadow-register to complete
    __no_operation(); // For setting breakpoint while debugging

    /* Read back the EEPROM registers into post_eeprom_write_regs array */
    for (i = 0; i < 24; i++)
    {
        if(!I2C_read32(eeprom_regmap[i][0], &(post_eeprom_write_regs[i]), I2C_TIMEOUT))
            while(1);
    }

    __no_operation();  // For setting breakpoint while debugging

    while (1);
}

bool I2C_write32(unsigned long addr, unsigned long writedata, uint32_t timeout) {
    EUSCI_B_I2C_initMasterParam param = {0};
    param.selectClockSource = EUSCI_B_I2C_CLOCKSOURCE_SMCLK;
    param.i2cClk = CS_getSMCLK();
    param.dataRate = EUSCI_B_I2C_SET_DATA_RATE_400KBPS;
    param.byteCounterThreshold = 0;
    param.autoSTOPGeneration = UCASTP_0;
    EUSCI_B_I2C_initMaster(EUSCI_I2C_BASE, &param);

    //Enable I2C Module to start operations
    EUSCI_B_I2C_enable(EUSCI_I2C_BASE);

    // Construct the 24 bit control word (refer to datasheet section 7.6.2.1)
    char control_word[3] = {0x10, 0x00, addr&0xFF};

    /* Set slave Address */
    EUSCI_B_I2C_setSlaveAddress(EUSCI_I2C_BASE,
                SLAVE_ADDR
            );

    /* Set master to transmit mode */
    EUSCI_B_I2C_setMode(EUSCI_I2C_BASE,
        EUSCI_B_I2C_TRANSMIT_MODE);

    EUSCI_B_I2C_masterSendStart(EUSCI_I2C_BASE);

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW23-CW16 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[0], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW15-CW8 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[1], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW7-CW0 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[2], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send D7-D0 of the 32-bit data */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, writedata & 0x000000FF, timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send D15-D8 of the 32-bit data */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, (writedata & 0x0000FF00)>>8, timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send D23-D16 of the 32-bit data */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, (writedata & 0x00FF0000)>>16, timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send D31-D23 of the 32-bit data */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, (writedata & 0xFF000000)>>24, timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    EUSCI_B_I2C_masterSendMultiByteStop(EUSCI_I2C_BASE);

    return 1;
}

bool I2C_read32(unsigned long addr, unsigned long *result, uint32_t timeout) {
    // Temp variables
    uint8_t val;
    unsigned long r = 0;

    EUSCI_B_I2C_initMasterParam param = {0};
    param.selectClockSource = EUSCI_B_I2C_CLOCKSOURCE_SMCLK;
    param.i2cClk = CS_getSMCLK();
    param.dataRate = EUSCI_B_I2C_SET_DATA_RATE_400KBPS;
    param.byteCounterThreshold = 4;
    param.autoSTOPGeneration = EUSCI_B_I2C_SEND_STOP_AUTOMATICALLY_ON_BYTECOUNT_THRESHOLD;
    EUSCI_B_I2C_initMaster(EUSCI_I2C_BASE, &param);

    //Enable I2C Module to start operations
    EUSCI_B_I2C_enable(EUSCI_I2C_BASE);

    // Construct the 24 bit control word (refer to datasheet section 7.6.2.1)
    char control_word[3] = {0x90, 0x00, addr&0xFF};

    /* Set slave Address */
    EUSCI_B_I2C_setSlaveAddress(EUSCI_I2C_BASE,
                SLAVE_ADDR
            );

    /* Set master to transmit mode */
    EUSCI_B_I2C_setMode(EUSCI_I2C_BASE,
        EUSCI_B_I2C_TRANSMIT_MODE);

    EUSCI_B_I2C_masterSendStart(EUSCI_I2C_BASE);

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW23-CW16 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[0], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW15-CW8 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[1], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /* Send bits CW7-CW0 of the control word */
    if (!EUSCI_B_I2C_masterSendMultiByteNextWithTimeout(EUSCI_I2C_BASE, control_word[2], timeout))
        return 0;

    __delay_cycles(INTER_BYTE_DELAY);

    /*
     * Set master to receive mode and generate Start condition
     * This sends out the slave address and continues to read
     * until you issue a STOP
     */
    EUSCI_B_I2C_masterReceiveStart(EUSCI_I2C_BASE);

    __delay_cycles(INTER_BYTE_DELAY);

    val = EUSCI_B_I2C_masterReceiveSingle(EUSCI_I2C_BASE);
    r |= val;
    __delay_cycles(INTER_BYTE_DELAY);

    val = EUSCI_B_I2C_masterReceiveSingle(EUSCI_I2C_BASE);
    r |= ((unsigned long)val)<<8;
    __delay_cycles(INTER_BYTE_DELAY);

    val = EUSCI_B_I2C_masterReceiveSingle(EUSCI_I2C_BASE);
    r |= ((unsigned long)val)<<16;
    __delay_cycles(INTER_BYTE_DELAY);

    val = EUSCI_B_I2C_masterReceiveSingle(EUSCI_I2C_BASE);
    r |= ((unsigned long)val)<<24;
    *result = r;
    __delay_cycles(INTER_BYTE_DELAY);

    return 1;
}
